party_config = {
    'artifactory_url': 'http://your-instance/artifactory/api',
    'search_prop': 'search/prop',
    'search_name': 'search/artifact',
    'search_repos': 'repositories',
    'username': 'your-user',
    'password': 'base64-encoded-password',
    'headers': {'Content-type': 'application/json'}
}
